/*!
 * @name        jQuery Modal
 * @author      Matt Hinchliffe <https://github.com/i-like-robots/jQuery-Modal>
 * @modified    2012-12-10
 * @version     1.0.1
 */
(function( $, undefined ) {

    'use strict';

    function Modal( target, options ) {
        this.opts = $.extend({}, {
            onopen: undefined,
            closeText: 'close',
            onhide: undefined,
            onupdate: undefined,
            fixed: false,
            overlay: true,
            blur: true,
            escape: true,
            wrapperClass: 'modal-wrapper',
            width: 640,
            maxWidth: '95%',
            height: 480,
            maxHeight: '95%',
            namespace: 'modal'
        }, options);

        this.target = target;

        this.isOpen = false;
        this.isInitialized = false;

        return this;
    }

    /**
     * Instantiate
     * @description Create the structure on first run
     */
    Modal.prototype._init = function() {

        console.log('function _init');
        this.doc = $(document);

        this.wrapper = $('<div class="'+this.opts.wrapperClass+'">').css({
            position: this.opts.fixed ? 'fixed' : 'absolute',
            width: this.opts.width,
            maxWidth: this.opts.maxWidth,
            height: this.opts.height,
            maxHeight: this.opts.maxHeight,
            display: 'none'
        });

        this.close = $('<span wrapper="'+this.opts.wrapperClass+'" class="modal-close" data-' + this.opts.namespace + '-close>'+this.opts.closeText+'</span>').appendTo(this.wrapper);
        this.content = $('<div class="modal-content">').appendTo(this.wrapper);

        this.wrapper.appendTo(this.target);

        // Define overlay to prevent errors
        this.overlay = false;

        if ( this.opts.overlay ) {
            this.overlay = $('<div class="over-'+this.opts.wrapperClass+' modal-overlay"' + (this.opts.blur ? 'data-' + this.opts.namespace + '-close' : '') + '>')
                .css({
                    position: 'absolute',
                    top: 0,
                    left: 0,
                    width: '100%',
                    display: 'none'
                })
                .appendTo(this.target);
        }

        // Context appropriate metrics and events
        this.isBody = (this.target === document.body);
        this.context = this.isBody ? $(window) : $(this.target).css('position', 'relative');

        this.isInitialized = true;
    };

    /**
     * Align
     * @description Centre modal window and size overlay to fit
     */
    Modal.prototype.align = function() {

        var height = this.wrapper.height(),
            width = this.wrapper.width(),
            maxHeight = this.context.height(),
            maxWidth = this.context.width(),
            top = this.opts.fixed ? 0 : this.context.scrollTop();

        this.wrapper.css({
            top: height < maxHeight ? ( (maxHeight - height ) / 2 ) + top : top,
            left: width < maxWidth ? (maxWidth - width) / 2 : 0
        });

        if ( this.opts.overlay ) {
            this.overlay.css('height', this.isBody ? this.doc.height() : maxHeight );
        }
    };

    /**
     * Open
     * @description Open the modal window
     * @param {object} content
     * @param {function} callback
     */
    Modal.prototype.open = function( content, callback ) {
        //if ( ! this.isInitialized) {
            this._init();
        //}

        var self = this;

        if ( this.isBody ) {
            this.context.on('resize.' + this.opts.namespace, function() {
                self.align();
            });

            this.doc.on('keyup.' + this.opts.namespace, function( e ) {
                if ( e.keyCode === 27 ) {
                    self.hide('',e);
                }
            });
        }

        this.doc.on('click.modal', '[data-' + this.opts.namespace + '-close]', function( e ) {
            e.preventDefault();
            self.hide('',e);
        });

        // Fade in
        this.wrapper
            .add(this.overlay)
            .stop()
            .fadeIn();

        this.isOpen = true;

        // Add content to window
        if ( content ) {
            this.update(content);
        }

        // Callbacks
        if ( this.opts.onopen ) {
            this.opts.onopen.call(this);
        }
        if ( callback ) {
            callback.call(this);
        }
    };

    /**
     * Update
     * @description Update the modal window contents
     * @param {object|string} content
     * @param {function} callback
     */
    Modal.prototype.update = function( content, callback ) {

        this.content.html(content);

        if ( this.isOpen ) {
            this.align();
        }

        // Callbacks
        if ( this.opts.onupdate ) {
            this.opts.onupdate.call(this);
        }
        if ( callback )
        {
            callback.call(this);
        }
    };

    /**
     * Resize
     * @description Resizes the modal window content area
     * @param {numeric|string} width
     * @param {numeric|string} height
     */
    Modal.prototype.resize = function( width, height ) {
        this.wrapper.css({
            width: width,
            height: height
        });

        this.align();
    };

    /**
     * Hide
     * @description Hide the modal window
     * @param {function} callback
     */
    Modal.prototype.hide = function( callback , e ) {

        // Unbind events
        var classModal = $(e.target).attr('wrapper');
        this.wrapper = $('.'+classModal);
        this.doc.off('.over-' + this.opts.namespace);
        this.overlay = $('div.over-'+classModal+'.modal-overlay');
        this.wrapper
            .add(this.overlay)
            .stop()
            .fadeOut().remove();

        this.isOpen = false;

        // Callbacks
        if ( this.opts.onhide ) {
            this.opts.onhide.call(this);
        }
        if ( callback ) {
            callback.call(this);
        }
    };

    // jQuery plugin wrapper
    $.fn.modal = function( options ) {
        return this.each(function() {
            if ( ! $.data(this, 'modal') ) {
                $.data(this, 'modal', new Modal(this, options) );
            }
        });
    };

})(jQuery);